﻿local copyCache = {}
local selectedName
local function eventHandler(self, event, ...)
  if (event=="VARIABLES_LOADED") then
  	BarCopyMainFrame:SetPoint("BOTTOMLEFT","UIParent", "BOTTOMLEFT", BarCopySettings.dispx, BarCopySettings.dispy)
	if not BarCopySettings.backups or BarCopySettings.backups == nil then
		BarCopySettings.backups = {}
	end
  end
end

function BarCopy_OnLoad(self)
	local version = GetAddOnMetadata("BarCopy", "Version")
	BarCopy_SetVariables()
	--One of them allows BarCopy to be closed with the Escape key
	tinsert(UISpecialFrames, "BarCopyMainFrame")
	UIPanelWindows["BarCopyMainFrame"] = nil

	self:SetScript("OnEvent", eventHandler)
	self:RegisterEvent("VARIABLES_LOADED")

	SlashCmdList["BarCopy"] = function(msg)
		BarCopy_SlashCommand(msg)
	end
	SLASH_BarCopy1 = "/barcopy"

	if( DEFAULT_CHAT_FRAME ) then
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff00BarCopy v"..version.." loaded")
	end
	UIErrorsFrame:AddMessage("BarCopy v"..version.." AddOn loaded", 1.0, 1.0, 1.0, 1.0, UIERRORS_HOLD_TIME)
end

function BarCopy_SetVariables()
	if not BarCopySettings then
		BarCopySettings = {}
		BarCopySettings.dispx = GetScreenWidth()/2 - 200
		BarCopySettings.dispy = GetScreenHeight()/2 - 200
		BarCopySettings.backups = {}
	end
end

function BarCopy_SlashCommand(msg)
  if(msg) then
	local command = strlower(msg)
	if(command == "show") then
		BarCopyMainFrame:Show()
	else
		if BarCopyMainFrame:IsShown() then
			BarCopyMainFrame:Hide()
		else
			BarCopyMainFrame:Show()
		end
	end
  end
end

function BarCopy_SavePosition(panel)
	BarCopySettings.dispx = panel:GetLeft()
	BarCopySettings.dispy = panel:GetBottom()
end

function BarCopy_Paste(bar)
	if BarCopyMainFrameBackupCheck:GetChecked() then
		local name = UIDropDownMenu_GetSelectedName(BarCopyMainFrameBackup)
		if name then
			copyCache = BarCopySettings.backups[name]
		else
			return
		end
		if not copyCache then
			return
		end
	end
	for i = 1, 12 do
		local action = (bar-1)*12 + i
		ClearCursor()
		local texture = _G["BarCopyMainFrameIndicator"..i]:GetNormalTexture()
		if texture then
			texture = texture:GetTexture()
		end
		if texture == "Interface\\Buttons\\UI-CheckBox-Check-Disabled" or not texture then
			PickupAction(action)
			ClearCursor()
		elseif texture == "Interface\\Buttons\\UI-CheckBox-Check" then
			--Do nothing
		else
			PickupAction(action)
			ClearCursor()
			if not copyCache or not copyCache[i] then
				--Do nothing
			elseif copyCache[i][1] == "spell" then
				PickupSpell(copyCache[i][2])
				if not GetCursorInfo() then
					local name = GetSpellInfo(copyCache[i][2])
					PickupSpellBookItem(name)
				end
			elseif copyCache[i][1] == "item" then
				PickupItem(copyCache[i][2])
			elseif copyCache[i][1] == "macro" then
				PickupMacro(copyCache[i][2])
			elseif copyCache[i][1] == "flyout" then
				BarCopy_PickupFlyout(copyCache[i][2])
			elseif copyCache[i][1] == "companion" then
				local position
				for a = 1, GetNumCompanions(copyCache[i][3]) do
					creatureID, creatureName, spellID = GetCompanionInfo(copyCache[i][3], a)
					if spellID == copyCache[i][2] then
						position = a
					end
				end
				PickupCompanion(copyCache[i][3],position)
			elseif copyCache[i][1] == "equipmentset" then
				PickupEquipmentSet(copyCache[i][2])
			end
			PlaceAction(action)
		end
	end
	ClearCursor()
end

function BarCopy_Copy(bar)
	copyCache = {}
	for i = 1, 12 do
		local type, id, subtype, spellId = GetActionInfo((bar-1)*12+i)
		copyCache[i] = {type, id, subtype, spellId}
		if (type == "flyout") then
			name = GetFlyoutInfo(id)
			copyCache[i][3] = name
		end
		local texture = GetActionTexture((bar-1)*12+i)
		local button = _G["BarCopyMainFrameIndicator"..i]
		button:SetNormalTexture(texture)
	end
	if BarCopyMainFrameBackupCheck:GetChecked() then
		local name = BarCopyMainFrameName:GetText()
		if name == "" then
			BarCopyMainFrameName:SetText("Choose name")
			return
		end
		if not BarCopySettings.backups then
			BarCopySettings.backups = {}
		end
		BarCopySettings.backups[name] = copyCache
		BarCopy_BackupSelect_Initialize()
	end
end

function BarCopy_PickupFlyout(id)
	local iii=1
	while true do
		local sbtype, sbid = GetSpellBookItemInfo(iii, "spell")
		if sbtype=="FLYOUT" and sbid==id then
			PickupSpellBookItem(iii, "spell")
			return true
		elseif sbtype==nil and sbid==nil then
			return false
		end
		iii = iii+1
	end
end

function BarCopyButton_OnClick(button)
	local thisID = button:GetID()
	local what
	if thisID > 20 then
		thisID = thisID - 20
		BarCopy_Paste(thisID)
	else
		thisID = thisID - 10
		BarCopy_Copy(thisID)
	end
end

function BarCopyCheckButton_OnClick(button)
	local i = tonumber(strmatch(button:GetName(), "%d+"))
	if button:GetNormalTexture() and button:GetNormalTexture():GetTexture() == "Interface\\Buttons\\UI-CheckBox-Check" then
		button:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled")
	elseif button:GetNormalTexture() and button:GetNormalTexture():GetTexture() == "Interface\\Buttons\\UI-CheckBox-Check-Disabled" then
		local newTexture
		if not copyCache or not copyCache[i] then
			newTexture = nil
		elseif copyCache[i][1] == "spell" then
			local name, rank, icon = GetSpellInfo(copyCache[i][2])
			newTexture = icon
		elseif copyCache[i][1] == "item" then
			local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, icon = GetItemInfo(copyCache[i][2])
			newTexture = icon
		elseif copyCache[i][1] == "macro" then
			local name, icon = GetMacroInfo(copyCache[i][2])
			newTexture = icon
		elseif copyCache[i][1] == "flyout" then
			local iii=1
			while true do
				local sbname, sbsubtext = GetSpellBookItemName(iii, "spell")
				if sbname==copyCache[i][3] then
					local name, rank, icon = GetSpellInfo(iii, "spell")
					newTexture = icon
					break
				elseif sbtype==nil and sbid==nil then
					break
				end
				iii = iii+1
			end
		elseif copyCache[i][1] == "companion" then
			local position
			for a = 1, GetNumCompanions(copyCache[i][3]) do
				local creatureID, creatureName, spellID, icon = GetCompanionInfo(copyCache[i][3], a)
				if spellID == copyCache[i][2] then
					newTexture = icon
					break
				end
			end
		elseif copyCache[i][1] == "equipmentset" then
			local name, icon = GetEquipmentSetInfo(copyCache[i][2])
			newTexture = icon
		end
		if not newTexture then
			newTexture = "Interface\\Buttons\\UI-EmptySlot"
		end
		button:SetNormalTexture(newTexture)
	else
		button:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Check")
	end
end

function BarCopy_BackupSelect_OnShow()
	UIDropDownMenu_Initialize(BarCopyMainFrameBackup, BarCopy_BackupSelect_Initialize)
	UIDropDownMenu_SetSelectedID(BarCopyMainFrameBackup, 1)
end

function BarCopy_BackupSelect_OnClick(button)
  local i = button:GetID()
  if not button:GetText() then
	for k,v in pairs(BarCopySettings.backups) do
		if not v or k == "" then
			BarCopySettings.backups[k] = nil
		end
	end
	BarCopy_BackupSelect_Initialize()
	return
  end
  selectedName = button:GetText()
  UIDropDownMenu_SetSelectedID(BarCopyMainFrameBackup, i)
  UIDropDownMenu_SetSelectedName(BarCopyMainFrameBackup, selectedName)
  BarCopyMainFrameName:SetText(selectedName)
  BarCopy_UpdateActionIndicators(selectedName)
end

function BarCopy_BackupSelect_Initialize()
	for k,v in pairs(BarCopySettings.backups) do
		info = {
			text = k,
			func = BarCopy_BackupSelect_OnClick
		}
		UIDropDownMenu_AddButton(info)
	end
end

function BarCopy_UpdateActionIndicators(selectedName)
	local selectedCache = BarCopySettings.backups[selectedName]
	for i = 1, 12 do
		local newTexture
		if not selectedCache or not selectedCache[i] then
			newTexture = nil
		elseif selectedCache[i][1] == "spell" then
			local name, rank, icon = GetSpellInfo(selectedCache[i][2])
			newTexture = icon
		elseif selectedCache[i][1] == "item" then
			local name, link, quality, iLevel, reqLevel, class, subclass, maxStack, equipSlot, icon = GetItemInfo(selectedCache[i][2])
			newTexture = icon
		elseif selectedCache[i][1] == "macro" then
			local name, icon = GetMacroInfo(selectedCache[i][2])
			newTexture = icon
		elseif selectedCache[i][1] == "flyout" then
			local iii=1
			print(selectedCache[i][3])
			while true do
				--local sbtype, spid = GetSpellBookItemInfo(iii, "spell")
				local sbname, sbsubtext = GetSpellBookItemName(iii, "spell")
				if sbname == selectedCache[i][3] then
				--if sbtype=="SPELL" and spid==selectedCache[i][2] then
					--local name, rank, icon = GetSpellInfo(iii, "spell")
					local icon = GetSpellBookItemTexture(iii, "spell")
					newTexture = icon
					break
				elseif sbtype==nil and sbid==nil then
					break
				end
				iii = iii+1
			end
		elseif selectedCache[i][1] == "companion" then
			local position
			for a = 1, GetNumCompanions(selectedCache[i][3]) do
				local creatureID, creatureName, spellID, icon = GetCompanionInfo(selectedCache[i][3], a)
				if spellID == selectedCache[i][2] then
					newTexture = icon
					break
				end
			end
		elseif selectedCache[i][1] == "equipmentset" then
			local name, icon = GetEquipmentSetInfo(selectedCache[i][2])
			newTexture = icon
		end
		if newTexture == nil then
			newTexture = "Interface\\Buttons\\UI-EmptySlot"
		end
		_G["BarCopyMainFrameIndicator"..i]:SetNormalTexture(newTexture)
	end
end

function BarCopy_Delete_OnClick()
	if BarCopyMainFrameName:GetText() and not(BarCopyMainFrameName:GetText() == "") then
		BarCopySettings.backups[BarCopyMainFrameName:GetText()] = nil
	end
	BarCopyMainFrameName:SetText("")
	BarCopy_BackupSelect_Initialize()
	UIDropDownMenu_SetSelectedID(BarCopyMainFrameBackup,0)
  UIDropDownMenu_SetSelectedName(BarCopyMainFrameBackup, "")
end